#include <QtCore/QCoreApplication>

// add Win32 headers here
#include <windows.h>

//#include <list>
#include <QList>
#include <QtAlgorithms>
#include <iostream>

// high resolution timer object
class THighResTimer
{
private:
  LARGE_INTEGER t0,t1,f;
  double et,last_dt;
  bool na;
protected:
public:
  THighResTimer(void)
  {
    QueryPerformanceFrequency(&f);
    na=f.QuadPart==0;
    HighResReset();
  }
  void HighResReset(void)   // reset timer
  {
    QueryPerformanceCounter(&t0);
    last_dt=0.0;
    et=0.0;
  }
  double HighResTime(void)	// elapsed time in seconds
  {
    double dt;

    if (na) return 0.0;

    QueryPerformanceCounter(&t1);
    dt=double(t1.QuadPart-t0.QuadPart);
    if (dt<0.0)
    {
      et=et+last_dt/f.QuadPart;
      t0=t1;
      last_dt=0.0;
      return et;
    }
    else
    {
      last_dt=dt;
      return et+dt/f.QuadPart;
    }
  }
};

int main(int argc, char *argv[])
{
  QCoreApplication a(argc, argv);

#ifdef _DEBUG
  static const int NT=1;
#else
  static const int NT=20;
#endif
  static const int ND=5000;

  using namespace std;

  QList <int> QList1;
  QList <int> QList2;
  double t;

  THighResTimer *HRT=new THighResTimer();

  for (int i=0;i<ND;i++)
  {
    QList1.push_back(ND-i);
  }

  HRT->HighResReset();
  for (int i=0;i<NT;i++)
  {
    QList<int>::iterator itr;
    for( itr = QList1.begin(); itr != QList1.end(); itr++)
    {
      QList2.push_back(*itr);
    }
    QList2.clear();
  }
  t=HRT->HighResTime()/NT;
  cout << "QList Add-Clear Time: " << t << "\r\n";

  HRT->HighResReset();
  for (int i=0;i<NT;i++)
  {
    QList<int>::iterator itr;
    for( itr = QList1.begin(); itr != QList1.end(); itr++)
    {
      QList2.push_back(*itr);
    }
    qSort(QList2.begin(),QList2.end());
    QList2.clear();
  }
  t=HRT->HighResTime()/NT;
  cout << "QList Add-Sort1-Clear Time: " << t << "\r\n";

  HRT->HighResReset();
  for (int i=0;i<NT;i++)
  {
    QList<int>::iterator itr = QList1.end();
    while (itr != QList1.begin())
    {
      itr--;
      QList2.push_back(*itr);
    }
    qSort(QList2.begin(),QList2.end());
    QList2.clear();
  }
  t=HRT->HighResTime()/NT;
  cout << "QList Add-Sort2-Clear Time: " << t << "\r\n";

  QList1.clear();

  delete HRT;

  return a.exec();
}
